/***************************************************
//Web: http://www.buydisplay.com
EastRising Technology Co.,LTD
Examples for ER-TFTM090-2 with Capacitive Touch Panel 
Display is Hardward SPI 4-Wire SPI Interface and 5V Power Supply
Capacitive Touch Panel is I2C Interface
Tested and worked with:
Works with Arduino 1.6.0 IDE  
Works with Arduino DUE,Arduino mega2560,Arduino UNO Board
Note: If you use our company's adapter board, you must remove the R6 resistor on the adapter board, otherwise the capacitive touch screen will not work.
****************************************************/
#include <stdint.h>
#include <SPI.h>
#include <Wire.h>

uint8_t addr  = 0x5d;  //CTP IIC ADDRESS

#include "Adafruit_GFX.h"
#include "Adafruit_RA8875.h"

//LCD:hardware SPI  CTP:hardware IIC
//Arduino DUE,Arduino mega2560,Arduino UNO
#define RA8875_INT 4
#define RA8875_CS 10 
#define RA8875_RESET 9
#define GT9271_INT   7     
Adafruit_RA8875 tft = Adafruit_RA8875(RA8875_CS, RA8875_RESET);


unsigned char  GTP_CFG_DATA[] =
{

0x00,0x20,0x03,0xE0,0x01,0x0A,0x0D,0x00,
0x01,0x0A,0x28,0x0F,0x50,0x32,0x03,0x08,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x8E,0x2E,0x88,0x23,0x21,
0x31,0x0D,0x00,0x00,0x00,0x01,0x03,0x1D,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x1E,0x50,0x94,0xC5,0x02,
0x07,0x00,0x00,0x04,0x80,0x21,0x00,0x6B,
0x28,0x00,0x59,0x31,0x00,0x4B,0x3B,0x00,
0x3F,0x48,0x00,0x3F,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x15,0x14,0x11,0x10,0x0F,0x0E,0x0D,0x0C,
0x09,0x08,0x07,0x06,0x05,0x04,0x01,0x00,
0xFF,0xFF,0xFF,0xFF,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x02,
0x04,0x06,0x07,0x08,0x0A,0x0C,0x0D,0x0F,
0x10,0x11,0x12,0x13,0x19,0x1B,0x1C,0x1E,
0x1F,0x20,0x21,0x22,0x23,0x24,0x25,0x26,
0x27,0x28,0xFF,0xFF,0xFF,0xFF,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x4D,0x01
};

struct TouchLocation
{
  uint16_t x;
  uint16_t y;
};
TouchLocation touchLocations[10];


void inttostr(uint16_t value,uint8_t *str);
uint8_t gt9271_Send_Cfg(uint8_t * buf,uint16_t cfg_len);
void writeGT9271TouchRegister( uint16_t regAddr,uint8_t *val, uint16_t cnt);
uint8_t readGT9271TouchAddr( uint16_t regAddr, uint8_t * pBuf, uint8_t len );
uint8_t readGT9271TouchLocation( TouchLocation * pLoc, uint8_t num );
uint32_t dist(const TouchLocation & loc);
uint32_t dist(const TouchLocation & loc1, const TouchLocation & loc2);
bool sameLoc( const TouchLocation & loc, const TouchLocation & loc2 );

uint8_t buf[30];


uint8_t gt9271_Send_Cfg(uint8_t * buf,uint16_t cfg_len)
{
	//uint8_t ret=0;
	uint8_t retry=0;
	for(retry=0;retry<5;retry++)
	{
		writeGT9271TouchRegister(0x8047,buf,cfg_len);
		//if(ret==0)break;
		delay(10);	 
	}
	//return ret;
}


void writeGT9271TouchRegister( uint16_t regAddr,uint8_t *val, uint16_t cnt)
{	uint16_t i=0;
  Wire.beginTransmission(addr);
   Wire.write( regAddr>>8 );  // register 0
  Wire.write( regAddr);  // register 0 
	for(i=0;i<cnt;i++,val++)//data
	{		
          Wire.write( *val );  // value
	}
  uint8_t retVal = Wire.endTransmission(); 
}



uint8_t readGT9271TouchAddr( uint16_t regAddr, uint8_t * pBuf, uint8_t len )
{
  Wire.beginTransmission(addr);
  Wire.write( regAddr>>8 );  // register 0
  Wire.write( regAddr);  // register 0  
  uint8_t retVal = Wire.endTransmission();
  
  uint8_t returned = Wire.requestFrom(addr, len);    // request 1 bytes from slave device #2
  
  uint8_t i;
  for (i = 0; (i < len) && Wire.available(); i++)
  
  {
    pBuf[i] = Wire.read();
  }
  
  return i;
}

uint8_t readGT9271TouchLocation( TouchLocation * pLoc, uint8_t num )
{
  uint8_t retVal;
  uint8_t i;
  uint8_t k;
  uint8_t  ss[1];
  do
  {  
    
    if (!pLoc) break; // must have a buffer
    if (!num)  break; // must be able to take at least one
     ss[0]=0;
      readGT9271TouchAddr( 0x814e, ss, 1);
      uint8_t status=ss[0];

    if ((status & 0x0f) == 0) break; // no points detected
    uint8_t hitPoints = status & 0x0f;
    
    Serial.print("number of hit points = ");
    Serial.println( hitPoints );
    
   uint8_t tbuf[32]; uint8_t tbuf1[32];uint8_t tbuf2[16];  
    readGT9271TouchAddr( 0x8150, tbuf, 32);
    readGT9271TouchAddr( 0x8150+32, tbuf1, 32);
    readGT9271TouchAddr( 0x8150+64, tbuf2,16);
    
      if(hitPoints<=4)
            {   
              for (k=0,i = 0; (i <  4*8)&&(k < num); k++, i += 8)
              {
                pLoc[k].x = tbuf[i+1] << 8 | tbuf[i+0];
                pLoc[k].y = tbuf[i+3] << 8 | tbuf[i+2];
              }   
            }
        if(hitPoints>4)   
            {  
               for (k=0,i = 0; (i <  4*8)&&(k < num); k++, i += 8)
              {
                pLoc[k].x = tbuf[i+1] << 8 | tbuf[i+0];
                pLoc[k].y = tbuf[i+3] << 8 | tbuf[i+2];
              }               
              
              for (k=4,i = 0; (i <  4*8)&&(k < num); k++, i += 8)
              {
                pLoc[k].x = tbuf1[i+1] << 8 | tbuf1[i+0];
                pLoc[k].y = tbuf1[i+3] << 8 | tbuf1[i+2];
              }   
            } 
            
          if(hitPoints>8)   
            {  
                for (k=0,i = 0; (i <  4*8)&&(k < num); k++, i += 8)
              {
                pLoc[k].x = tbuf[i+1] << 8 | tbuf[i+0];
                pLoc[k].y = tbuf[i+3] << 8 | tbuf[i+2];
              }               
              
              for (k=4,i = 0; (i <  4*8)&&(k < num); k++, i += 8)
              {
                pLoc[k].x = tbuf1[i+1] << 8 | tbuf1[i+0];
                pLoc[k].y = tbuf1[i+3] << 8 | tbuf1[i+2];
              }                          
             
              for (k=8,i = 0; (i <  2*8)&&(k < num); k++, i += 8)
              {
                pLoc[k].x = tbuf2[i+1] << 8 | tbuf2[i+0];
                pLoc[k].y = tbuf2[i+3] << 8 | tbuf2[i+2];
              }   
            }       

            
    
    retVal = hitPoints;
    
  } while (0);
  
    ss[0]=0;
    writeGT9271TouchRegister( 0x814e,ss,1); 
  
  return retVal;
}


void setup() 
{
  Serial.begin(9600);
  Serial.println("RA8875 start");
  Wire.begin();        // join i2c bus (address optional for master)


     pinMode(RA8875_RESET, OUTPUT); 
    pinMode     (GT9271_INT, OUTPUT);
    digitalWrite(RA8875_RESET, LOW);
    digitalWrite(GT9271_INT, LOW);
    delay(2);
     digitalWrite(RA8875_RESET, HIGH);
     delay(20);  
    pinMode     (GT9271_INT, INPUT);
     delay(50);  
   
    uint8_t re=gt9271_Send_Cfg((uint8_t*)GTP_CFG_DATA,sizeof(GTP_CFG_DATA));
    

  while (!tft.begin(RA8875_800x480)) 
  {
    Serial.println("RA8875 Not Found!");
    delay(100);
  }
  Serial.println("Found RA8875");
  

  tft.displayOn(true);
  tft.GPIOX(true);      // Enable TFT - display enable tied to GPIOX
  tft.PWM1config(true, RA8875_PWM_CLK_DIV1024); // PWM output for backlight
  tft.PWM1out(255);

  // With hardware accelleration this is instant
  tft.fillScreen(RA8875_WHITE);

      tft.textMode();
      tft.textSetCursor(0,0);
      tft.textColor(RA8875_RED, RA8875_BLACK);
      tft.textWrite("www.buydiplay.com  Backlight brightness adjustment test ");
      tft.graphicsMode();

  // Play with PWM
  for (uint8_t i=255; i!=0; i-=5 ) 
  {
    tft.PWM1out(i); 
    delay(10);
  }  
  for (uint8_t i=0; i!=255; i+=5 ) 
  {
    tft.PWM1out(i); 
    delay(10);
  }
  tft.PWM1out(255); 
  
  tft.fillScreen(RA8875_RED);
  delay(500);
  tft.fillScreen(RA8875_YELLOW);
  delay(500);
  tft.fillScreen(RA8875_GREEN);
  delay(500);
  tft.fillScreen(RA8875_CYAN);
  delay(500);
  tft.fillScreen(RA8875_MAGENTA);
  delay(500);
  tft.fillScreen(RA8875_BLACK);
  
  // Try some GFX acceleration!
  tft.drawCircle(100, 100, 50, RA8875_BLACK);
  tft.fillCircle(100, 100, 49, RA8875_GREEN);
  
  tft.fillRect(11, 11, 700, 400, RA8875_BLUE);
  tft.drawRect(10, 10, 400, 200, RA8875_GREEN);
  tft.fillRoundRect(200, 10, 200, 100, 10, RA8875_RED);
  tft.fillRoundRect(480, 100, 300, 200, 10, RA8875_RED);
  tft.drawPixel(10,10,RA8875_BLACK);
  tft.drawPixel(11,11,RA8875_BLACK);
  //tft.drawLine(10, 10, 200, 100, RA8875_RED);
  tft.drawTriangle(200, 15, 250, 100, 150, 125, RA8875_BLACK);
  //tft.fillTriangle(200, 16, 249, 99, 151, 460, RA8875_YELLOW);
  tft.drawEllipse(300, 100, 100, 40, RA8875_BLACK);
  tft.fillEllipse(300, 100, 98, 38, RA8875_GREEN);
  // Argument 5 (curvePart) is a 2-bit value to control each corner (select 0, 1, 2, or 3)
  tft.drawCurve(50, 100, 80, 40, 2, RA8875_BLACK);  
  tft.fillCurve(50, 100, 78, 38, 2, RA8875_WHITE);
  
  
  tft.touchEnable(false);
    
  Serial.print("Status: "); Serial.println(tft.readStatus(), HEX);
  Serial.println("Waiting for touch events ...");
  
  randomSeed(analogRead(0));
}

uint32_t dist(const TouchLocation & loc)
{
  uint32_t retVal = 0;
  
  uint32_t x = loc.x;
  uint32_t y = loc.y;
  
  retVal = x*x + y*y;
  
  return retVal;
}

uint32_t dist(const TouchLocation & loc1, const TouchLocation & loc2)
{
  uint32_t retVal = 0;
  
  uint32_t x = loc1.x - loc2.x;
  uint32_t y = loc1.y - loc2.y;
  
  retVal = sqrt(x*x + y*y);
  
  return retVal;
}

bool sameLoc( const TouchLocation & loc, const TouchLocation & loc2 )
{
  return dist(loc,loc2) < 50;
}

void loop() 
{
  static uint16_t total = 256;
  static uint16_t w = tft.width();
  static uint16_t h = tft.height();
  
  float xScale = 1024.0F/w;
  float yScale = 1024.0F/h;
 


  uint8_t i=0;   
  

    while(total!=0)
  {
    total--;
    if ((total & 0x3ff) == 0)
    {
      //Serial.println(total);
    }
    // void    fillRect(int16_t x, int16_t y, int16_t w, int16_t h, uint16_t color);
    int16_t x  = random(w);
    int16_t y  = random(h);
    int16_t x1 = random(w);
    int16_t y1 = random(h);
    int16_t x2 = random(w);
    int16_t y2 = random(h);
    uint16_t c  = (((random(64)<<6)|random(64))<<6)|random(31);
    uint16_t bg = (((random(64)<<6)|random(64))<<6)|random(31);
    int16_t th = random(h);
    int16_t tw = random(w);
    int16_t op = random(9);
    int16_t en = random(8);
    
    switch( op )
    {
    default:
    case 0:
      tft.fillRect(x,y, tw, th, c);
      break;
    case 1:
      tft.fillEllipse(x,y,tw/2,th/2,c);
      break;
    case 2:
      tft.fillCircle(x,y,tw/2,c);
      break;
    case 3:
      tft.fillTriangle(x,y,x1,y1,x2,y2,c);
      break;
    case 4:
      tft.textMode();
      tft.textSetCursor(x,y);
      tft.textColor(c,bg);
      tft.textEnlarge(en);
      tft.textWrite("Hallowell",9);
      tft.graphicsMode();
      break;
    case 5:
      tft.drawRect(x,y, tw, th, c);
      break;
    case 6:
      tft.drawEllipse(x,y,tw/2,th/2,c);
      break;
    case 7:
      tft.drawCircle(x,y,tw/2,c);
      break;
    case 8:
      tft.drawTriangle(x,y,x1,y1,x2,y2,c);
      break;
    }
     if(total==0)
       {  tft.fillScreen(RA8875_BLACK);
         tft.textMode();
        tft.textSetCursor(0,0);
        tft.textColor(RA8875_RED, RA8875_BLACK);
         tft.textEnlarge(0);
        tft.textWrite("www.buydiplay.com  Capacitive touch screen test.Please touch the screen!");
              tft.textSetCursor(w-40,0); 
         tft.textWrite("CLEAR");     
       }      
  }
  

         pinMode     (GT9271_INT, INPUT);
       uint8_t st=digitalRead(GT9271_INT); 


     /* Wait around for touch events */
    if(!st) 
    {   
      Serial.println("Touch: ");
      
      uint8_t count = readGT9271TouchLocation( touchLocations, 10 );
      
      //static uint8_t lastCount = count;
  
     if (count)
      {
      /*  static TouchLocation lastTouch = touchLocations[0];
        
        if (((thisTouchTime - lastTouchTime) > 10000) && sameLoc( touchLocations[0],lastTouch ) )
        {
          tft.fillScreen(RA8875_BLACK);
          lastTouchTime = thisTouchTime; 
   
        }
      
        Serial.print("Time delta = ");
        Serial.print(thisTouchTime - lastTouchTime);
        Serial.print(", dist = ");
        Serial.println( dist(touchLocations[0],lastTouch) );
        
        lastTouch = touchLocations[0];
        
        Serial.println("Locations: ");*/
        
        for (i = 0; i < count; i++)
        {
          //Serial.print(i);
          //Serial.print(": ");
          //Serial.print(touchLocations[i].x);
          //Serial.print(", ");
          //Serial.print(touchLocations[i].y);
          //Serial.println(".");
          tft.textMode();
        tft.textSetCursor(0,0);
        tft.textColor(RA8875_RED, RA8875_BLACK);
         tft.textEnlarge(0);
      tft.textWrite("www.buydiplay.com  Capacitive touch screen test.Please touch the screen!");
          //tft.textSetCursor(touchLocations[i].x,touchLocations[i].y);
          tft.textSetCursor(380,200+16*i);
          tft.textColor(RA8875_WHITE, RA8875_BLACK);
          tft.textEnlarge(0);
          snprintf((char*)buf,sizeof(buf),"(%3d,%3d)",touchLocations[i].x,touchLocations[i].y);
          tft.textWrite((const char *)buf,9);
        tft.graphicsMode();  
        if(i==0)  tft.fillCircle(touchLocations[i].x,touchLocations[i].y, 3, RA8875_RED);  
        else if(i==1)  tft.fillCircle(touchLocations[i].x,touchLocations[i].y, 3, RA8875_GREEN); 
        else if(i==2)  tft.fillCircle(touchLocations[i].x,touchLocations[i].y, 3, RA8875_BLUE);        
        else if(i==3)  tft.fillCircle(touchLocations[i].x,touchLocations[i].y, 3, RA8875_CYAN); 
         else if(i==4)  tft.fillCircle(touchLocations[i].x,touchLocations[i].y, 3, RA8875_MAGENTA);
          else if(i==5)  tft.fillCircle(touchLocations[i].x,touchLocations[i].y, 3, RA8875_YELLOW); 
          else if(i==6)  tft.fillCircle(touchLocations[i].x,touchLocations[i].y, 3, 0xfc10); 
           else if(i==7)  tft.fillCircle(touchLocations[i].x,touchLocations[i].y, 3, 0x87f0);
           else if(i==8)  tft.fillCircle(touchLocations[i].x,touchLocations[i].y, 3, 0x841f);
          else if(i==9)  tft.fillCircle(touchLocations[i].x,touchLocations[i].y, 3, 0x87ff);   
         tft.textMode();     
        }
      }
        
          if(touchLocations[0].x>=760&&(touchLocations[0].y&0x0fff)<=30)
          {tft.fillScreen(RA8875_BLACK);
           tft.textMode();
          tft.textSetCursor(0,0);
          tft.textColor(RA8875_RED, RA8875_BLACK);
          tft.textEnlarge(0);
          tft.textWrite("www.buydiplay.com  Capacitive touch screen test.Please touch the screen!");  
           tft.textSetCursor(w-40,0); 
           tft.textWrite("CLEAR");  
          } 
    }
  
    else
    {
    }
    
   
}
